#include <stdio.h>
#include <stdlib.h>

#include "list.h"

#define MAX 10

int main(int argc, char** argv)
   {
   struct list * l = list_new();
   if (NULL == l)
      {
      perror("list_new");
      exit(EXIT_FAILURE);
      }


   /////////////////////////////////////
   // Push and pop from back
   for (int i = 0; i < MAX ; i++)
      {
      int * ip = malloc(sizeof(int));
      if (NULL == ip)
	 {
	 perror("malloc");
	 exit(EXIT_FAILURE);
	 }

      *ip = i;
      if (NULL == list_push_back(l, ip))
	 {
	 perror("list_push_back");
	 exit(EXIT_FAILURE);
	 }
      }

   printf("size %zd\n", list_size(l));
   while (0 != list_size(l))
      {
      int * ip = list_pop_back(l);
      printf("%d\n", *ip);
      free(ip);
      }

   /////////////////////////////////////
   // Push and pop from front
   for (int i = 0; i < MAX ; i++)
      {
      int * ip = malloc(sizeof(int));
      if (NULL == ip)
	 {
	 perror("malloc");
	 exit(EXIT_FAILURE);
	 }

      *ip = i;
      if (NULL == list_push_front(l, ip))
	 {
	 perror("list_push_front");
	 exit(EXIT_FAILURE);
	 }
      }

   printf("size %zd\n", list_size(l));
   while (0 != list_size(l))
      {
      int * ip = list_pop_front(l);
      printf("%d\n", *ip);
      free(ip);
      }


   /////////////////////////////////////
   // Push back and pop back
   for (int i = 0; i < MAX ; i++)
      {
      int * ip = malloc(sizeof(int));
      if (NULL == ip)
	 {
	 perror("malloc");
	 exit(EXIT_FAILURE);
	 }

      *ip = i;
      if (NULL == list_push_back(l, ip))
	 {
	 perror("list_push_back");
	 exit(EXIT_FAILURE);
	 }
      }

   printf("size %zd\n", list_size(l));
   while (0 != list_size(l))
      {
      int * ip = list_pop_front(l);
      printf("%d\n", *ip);
      free(ip);
      }

   /////////////////////////////////////
   // push back, walk front to back
   for (int i = 0; i < MAX ; i++)
      {
      int * ip = malloc(sizeof(int));
      if (NULL == ip)
	 {
	 perror("malloc");
	 exit(EXIT_FAILURE);
	 }

      *ip = i;
      if (NULL == list_push_back(l, ip))
	 {
	 perror("list_push_back");
	 exit(EXIT_FAILURE);
	 }
      }

   printf("size %zd\n", list_size(l));

   struct list_iterator * lit;
   int * ip;

   printf("Walking front to back\n");
   for (lit = list_begin(l); NULL != ip; ip = list_next( &lit))
      printf("%d\n", *ip);

   printf("Inserting 15 after 5\n");
   for (ip = list_begin(l, &lit); NULL != ip; ip = list_next( &lit))
      {
      if (5 == *ip)
	 {
	 int * newip = malloc(sizeof(int));
	 if (NULL == newip)
	    {
	    perror("malloc");
	    exit(EXIT_FAILURE);
	    }
	 *newip = 15;
	 if (NULL == list_insert_after(&lit, newip))
	    {
	    perror("malloc");
	    exit(EXIT_FAILURE);
	    }
	 break;
	 }
      }

   printf("Inserting 20 before 8\n");
   for (ip = list_begin(l, &lit); NULL != ip; ip = list_next( &lit))
      {
      if (8 == *ip)
	 {
	 int * newip = malloc(sizeof(int));
	 if (NULL == newip)
	    {
	    perror("malloc");
	    exit(EXIT_FAILURE);
	    }
	 *newip = 20;
	 if (NULL == list_insert_before(&lit, newip))
	    {
	    perror("malloc");
	    exit(EXIT_FAILURE);
	    }
	 break;
	 }
      }

   printf("size %zd\n", list_size(l));

   printf("Walking back to front\n");
   for (ip = list_end(l, &lit); NULL != ip ; ip = list_prev(&lit))
      printf("%d\n", *ip);


   printf("Erasing front to back\n");
   list_begin(l, &lit);
   while (NULL != (ip = list_value(&lit)))
      {
      printf("%d\n", *ip);
      list_erase(&lit);
      free(ip);
      }
   printf("size %zd\n", list_size(l));

   list_delete(l);
   exit(EXIT_SUCCESS);

   }
